"""
This is a file that has the following implemented.

# allows for the creation of a new user who wants to submit agents for benchmarking
register_user(username, password)

# allows for the creation of a new agent
register_agent(username, password, agent_name, code_path)

# allows for querying of benchmarks so users can easily choose what benchmarks they want to run
get_benchmark_ids(category=[], name=None, version='latest')

# starts the process of running a benchmark with the given id when this returns the agent can start working on the code
start_benchmark(id)

# allows the agent to ask a clarifying question before starting work on a ticket
ask_question(ticket_id, question)

# called when the agent is ready to submit the artifact. This will cause the code to be pushed to our git repo
submit_artifact(workspace: Path)
"""


from pathlib import Path
import time


import openapi_client
from openapi_client.apis.tags import default_api
from openapi_client.model.user import User
from openapi_client.model.create_user_request import CreateUserRequest
from openapi_client.model.errors_response import ErrorsResponse
from openapi_client import models
from pprint import pprint
from agent_harness.api_comms import (
    api_register_agent,
    handle_bids,
    upload_artifact,
    get_agents,
)


class PythonClientUser:
    def __init__(self, username: str, password: str, api_host: str):
        self.username = username
        self.password = password
        self.cfg = openapi_client.Configuration(
            host=api_host,
            username=self.username,
            password=self.password,
        )
        self.api = openapi_client.ApiClient(self.cfg)
        self.instance = default_api.DefaultApi(self.api)


def register_user(
    username: str,
    password: str,
    email: str,
    api_host: str = "https://marketplace-api.ai-maintainer.com/v1",
) -> PythonClientUser:
    """
    Allows for the creation of a new user who wants to submit agents for benchmarking.

    Args:
        username (str): The username of the user.
        password (str): The password for the user.

    Returns:
        None
    """
    # CLIENT_USERNAME = "test_user1"
    # CLIENT_PASSWORD = "F@k3awefawefawef"
    client = PythonClientUser(username, password, api_host)

    # create user. expect 201 or 409
    req = models.CreateUserRequest(userName=username, password=password, email=email)
    try:
        response = client.instance.create_user(req)
        assert response.response.status == 201
        return client
    except openapi_client.exceptions.ApiException as e:
        assert e.status == 409


def fetch_users_agents(client) -> list:
    """
    Fetches all agents for a given user.

    Args:
        username (str): The username of the user.
        password (str): The password for the user.

    Returns:
        list: A list of agents.
    """
    agents = get_agents(client)
    return agents


def register_agent(client, agent_name: str) -> str:
    """
    Allows for the creation of a new agent.

    Args:
        username (str): The username of the user.
        password (str): The password for the user.
        agent_name (str): The name of the agent being registered.
        code_path (str): The path to the agent's code.

    Returns:
        str: agent_id
    """
    agents = get_agents(client)
    for agent in agents:
        if agent["agentName"] == agent_name:
            if agent["userName"] == client.username:
                raise ValueError("User already has an agent with this name.")
            else:
                raise PermissionError(
                    f"Agent name {agent_name} is already taken by another user."
                )
    return api_register_agent(client, agent_name)


def get_benchmark_ids(
    client, category: list = [], name: str = None, version: str = "latest"
) -> list:
    """
    Allows for querying of benchmarks so users can easily choose what benchmarks they want to run.

    Args:
        category (list, optional): The category of benchmarks. Defaults to [].
        name (str, optional): The name of the benchmark. Defaults to None.
        version (str, optional): The version of the benchmark. Defaults to 'latest'.

    Returns:
        list: A list of benchmark IDs.
    """
    return ["c62e6410-2a95-4392-a21c-5f9a6a067230"]


def start_benchmark(client, id: int, code_path: Path, agent_id: str) -> None:
    """
    Starts the process of running a benchmark with the given id. When this returns, the agent can start working on the code.

    Args:
        id (int): The ID of the benchmark.
        code_path (Path): The path where code can be dumped into the workspace for the agent to start work.

    Returns:
        None
    """
    req = models.CreateBenchmarkTicketRequest(
        agentId=agent_id,
        benchmarkId=id,
    )
    response = client.instance.create_benchmark_ticket(req)
    while True:
        # poll for tickets assigned to this user
        response = client.instance.get_agent_tickets(
            query_params={
                "agentId": agent_id,
            }
        )
        tickets = list(response.body["tickets"])
        print("tickets:", tickets)
        if len(tickets) == 0:
            print("No tickets found. Sleeping.")
            time.sleep(2)
            continue
        ticket_id = tickets[0]["ticketId"]

        # create bid
        req = models.CreateBidRequest(
            agentId=agent_id,
            ticketId=ticket_id,
            rate=0.0,
        )
        response = client.instance.create_bid(req)
        print("response.body:", response.body)

        while True:
            # wait for the bids to be accepted.
            fork, bid_id, ticket = handle_bids(client, agent_id, code_path)
            print("fork:", fork)
            if fork:
                return fork, bid_id, ticket
            time.sleep(0.5)


def ask_question(ticket_id: int, question: str) -> None:
    """
    Allows the agent to ask a clarifying question before starting work on a ticket.

    Args:
        ticket_id (int): The ID of t    he ticket.
        question (str): The question being asked.

    Returns:
        None
    """
    return "No"


def submit_artifact(client, fork, bid_id: str, path: Path) -> None:
    """
    Called when the agent is ready to submit the artifact. This will cause the code to be pushed to our git repo.

    Args:
        workspace (Path): The path to the workspace containing the artifact.

    Returns:
        None
    """
    upload_artifact(client, fork, bid_id, path)



import pytest
import os
import subprocess
from pathlib import Path
import shutil

from agent_harness.aider_config.aider_interface import (
    _get_python_files,
    register_agent,
    start_agent_task,
)

from tests.test_agents.verification_python.sum_checker import check_two_sum
from tests.test_agents.test_python.sum import two_sum


@pytest.fixture
def test_data_dir(tmp_path):
    test_dir = tmp_path / "test_data"
    test_dir.mkdir(exist_ok=True)

    # Ensure the directory is empty
    for file_name in os.listdir(test_dir):
        file_path = os.path.join(test_dir, file_name)
        if os.path.isfile(file_path):
            os.unlink(file_path)
        elif os.path.isdir(file_path):
            shutil.rmtree(file_path)

    # Add some Python files to the root directory
    for i in range(3):
        with open(test_dir / f"file{i}.py", "w") as file:
            file.write(f"print('file{i}')")

    # Create nested directories and add Python files
    nested_dir1 = test_dir / "nested1"
    nested_dir1.mkdir()
    with open(nested_dir1 / "nested_file.py", "w") as file:
        file.write("print('nested_file')")

    nested_dir2 = nested_dir1 / "nested2"
    nested_dir2.mkdir()
    with open(nested_dir2 / "deeply_nested_file.py", "w") as file:
        file.write("print('deeply_nested_file')")

    return test_dir


def test_get_python_files(test_data_dir):
    mount_point = "/code"
    code_path = str(test_data_dir)

    expected_files = [f"{mount_point}/file{i}.py" for i in range(3)] + [
        f"{mount_point}/nested1/nested_file.py",
        f"{mount_point}/nested1/nested2/deeply_nested_file.py",
    ]

    actual_files = _get_python_files(mount_point, code_path)

    assert sorted(actual_files) == sorted(
        expected_files
    ), f"Expected {expected_files}, but got {actual_files}"


def test_aider_can_change_code():
    # Get the directory of the current file
    current_dir = Path(os.path.dirname(os.path.abspath(__file__)))

    # Define the code path and register the agent
    code_path = current_dir / "test_python"
    register_agent(code_path, current_dir.parent.parent)

    try:
        check_two_sum()
        # this should raise an assertion error if it doesn't, then we can't test the agent
        raise ValueError("The test should fail before the code is changed")
    except AssertionError:
        # The test should fail
        pass

    # Suppose the task text describes the bug that needs to be fixed
    task_text = f"Fix the `two_sum` function so that it adds values correctly."

    # Perform the task
    start_agent_task(task_text)

    # Run the same test again and assert that it now passes
    check_two_sum()



import subprocess
import base64
import os
import tempfile


class GitRepo:
    def __init__(self, url, username, password):
        self.url = url
        self.username = username
        self.password = password
        self._credentials = f"{self.username}:{self.password}"
        self._encoded_credentials = base64.b64encode(
            self._credentials.encode("utf-8")
        ).decode()
        self.auth_header = (
            f"http.extraHeader=Authorization: Basic {self._encoded_credentials}"
        )

    def print_std_streams(self, result):
        if result:
            if result.stderr:
                print(result.stderr.decode())
            if result.stdout:
                print(result.stdout.decode())

    def clone(self, directory):
        command = ["git", "-c", self.auth_header, "clone", self.url, directory]
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def fetch(self, directory):
        command = ["git", "-C", directory, "-c", self.auth_header, "fetch"]
        print(" ".join(command))
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def pull(self, directory, branch=None):
        command = ["git", "-C", directory, "-c", self.auth_header, "pull", "origin"]
        if branch:
            command.append(branch)
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def push(self, directory, branch=None, force=False):
        command = ["git", "-C", directory, "-c", self.auth_header, "push", "origin"]
        if branch:
            command.append(branch)
        if force:
            command.append("--force")
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def checkout(self, directory, branch=None):
        if not branch:
            raise ValueError("Branch name must be provided for checkout.")
        command = ["git", "-C", directory, "checkout", branch]
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def create_branch(self, directory, branch=None):
        if not branch:
            raise ValueError("Branch name must be provided for creating a branch.")
        command = ["git", "-C", directory, "checkout", "-b", branch]
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def merge(self, directory, branch=None):
        if not branch:
            raise ValueError("Branch name must be provided for merge.")
        command = ["git", "-C", directory, "merge", branch]
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def add(self, directory, files=".", all=False):
        command = ["git", "-C", directory, "add"]
        if all:
            command.append("--all")
        else:
            command.append(files)
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def commit(self, directory, message):
        if not message:
            raise ValueError("Commit message must be provided.")

        command = ["git", "-C", directory, "commit", "-m", message]
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        self.print_std_streams(result)

    def has_changes(self, directory):
        command = ["git", "-C", directory, "status", "--porcelain"]
        result = subprocess.run(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )

        output = result.stdout.decode().strip()

        # If the output is empty, there are no changes to commit
        if output:
            print("There are changes to commit.")
            return True
        else:
            print("There are no changes to commit.")
            return False

    def fork(self, fork_url, force=False):
        # Create a temporary directory
        with tempfile.TemporaryDirectory() as temp_dir:
            # Clone the original repository into the temporary directory
            self.clone(temp_dir)

            # Change to the temporary directory
            command = ["git", "-C", temp_dir, "remote", "add", "fork", fork_url]
            result = subprocess.run(
                command,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
            )

            # Push the contents to the fork URL, with force if specified
            push_command = [
                "git",
                "-C",
                temp_dir,
                "-c",
                self.auth_header,
                "push",
                "fork",
                "--all",
            ]
            if force:
                push_command.append("--force")

            result = subprocess.run(
                push_command,
                # stdout=subprocess.PIPE,
                # stderr=subprocess.PIPE,
                check=True,
            )
            self.print_std_streams(result)

        # The temporary directory and its contents will be deleted when the context is exited


def create_url(host: str, owner: str, repo: str) -> str:
    return f"{host}/{owner}/{repo}"


if __name__ == "__main__":
    GIT_HOST = "http://git-server:8080"
    CLIENT_USERNAME = "test_user1"
    CLIENT_PASSWORD = "F@k3awefawefawef"
    CLIENT_CODE_OWNER = CLIENT_USERNAME
    CLIENT_CODE_REPO = "repo1"
    OPERATOR_USERNAME = "test_user2"
    OPERATOR_PASSWORD = "F@k3awefawefawef"
    OPERATOR_CODE_OWNER = OPERATOR_USERNAME
    OPERATOR_CODE_REPO = "repo2"
    TEMP_DIR = "./deleteme"

    url = create_url(GIT_HOST, OPERATOR_CODE_OWNER, OPERATOR_CODE_REPO)
    gitrepo = GitRepo(
        url,
        OPERATOR_USERNAME,
        OPERATOR_PASSWORD,
    )
    path = os.path.join(TEMP_DIR, CLIENT_CODE_REPO)
    gitrepo.clone(path)



import json
import pytest
from pathlib import Path
import importlib


def check_two_sum():
    my_sum_module = importlib.import_module("tests.test_agents.test_python.sum")
    importlib.reload(my_sum_module)
    two_sum = my_sum_module.two_sum
    assert two_sum(1, 2) == 3
    assert two_sum(2, 2) == 4
    assert two_sum(0, 0) == 0
    assert two_sum(-1, 1) == 0
    assert two_sum(-1, -1) == -2



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = [
    "Click>=7.0",
    "agent-marketplace-sdk==0.1.3",
    "docker>=4.2.2",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author="Douglas Schonholtz",
    author_email="douglas@ai-maintainer.com",
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="AI Maintainer Agent Harness for our benchmarking and Marketplace API and platform",
    entry_points={
        "console_scripts": [
            "agent_harness=agent_harness.cli:main",
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="agent_harness",
    name="agent_harness",
    packages=find_packages(include=["agent_harness", "agent_harness.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/dschonholtz/agent_harness",
    version="0.1.5",
    zip_safe=False,
)
